import React, { useEffect, useState, useCallback } from 'react';
import Info from '../../components/info';
import { addVisualization } from '../../data-access/store/visualizationSlice';
import { useActiveSaveStateAuthorization, useAppDispatch } from '../../data-access';
import { Visualizations } from '../components/VisualizationPanel';

type VisualizationDescription = {
  name: string;
  displayName: string;
  description: string;
};

const displayNames: { [key: string]: string } = {
  ArcPlot: 'ArcPlot',
  MapVis: 'MapVis',
  MatrixVis: 'MatrixVis',
  NodeLinkVis: 'NodeLinkVis',
  PaohVis: 'PaohVis',
  TableVis: 'TableVis',
  SemanticSubstratesVis: 'SemanticSubstratesVis',
  RawJSONVis: 'RawJSONVis',
  Vis0D: 'Vis0D',
  Vis1D: 'Vis1D',
};

const descriptions: { [key: string]: string } = {
  ArcPlot: '',
  MapVis: 'Geographical Features',
  MatrixVis: 'Overview & Details',
  NodeLinkVis: 'General Patterns and Connections',
  PaohVis: 'Paths and Connection',
  TableVis: 'Node Attribute Statistics and Details',
  SemanticSubstratesVis: 'Node/Edge Attribute Exploration',
  RawJSONVis: '(Raw) Data Export',
  Vis0D: 'KPI visualization',
  Vis1D: 'Basic charts',
};

export function Recommender() {
  const dispatch = useAppDispatch();
  const saveStateAuthorization = useActiveSaveStateAuthorization();
  const [visualizationDescriptions, setVisualizationDescriptions] = useState<VisualizationDescription[]>([]);

  useEffect(() => {
    const loadVisualizations = async () => {
      const visDescriptions = await Promise.all(
        Object.keys(Visualizations).map((key) => {
          return {
            name: key,
            displayName: displayNames[key],
            description: descriptions[key],
          };
        }),
      );
      setVisualizationDescriptions(visDescriptions);
    };

    loadVisualizations();
  }, []);

  return (
    <div className="p-4">
      <span className="text-md">Select a visualization</span>
      <div className="grid grid-cols-3 gap-4 my-2">
        {visualizationDescriptions.map(({ name, displayName, description }) => (
          <div
            key={name}
            className={`p-4 border ${saveStateAuthorization.visualization.W ? 'cursor-pointer hover:bg-secondary-100' : 'cursor-not-allowed opacity-50'}`}
            onClick={async (e) => {
              e.preventDefault();
              if (!saveStateAuthorization.visualization.W) {
                console.debug('User blocked from editing query due to being a viewer');
                return;
              }

              const component = await Visualizations[name]();
              dispatch(addVisualization({ ...component.default.settings, name: name, id: name }));
            }}
          >
            <div className="flex items-center justify-between">
              <span className="text-sm font-semibold truncate">{displayName}</span>
              <Info tooltip="Here an explanation" placement="top" />
            </div>
            <div className="flex items-center justify-between">
              <span className="text-xs text-secondary-400 truncate">{description}</span>
            </div>
          </div>
        ))}
      </div>
    </div>
  );
}
